package com.example.workflow.rest.controller;

import org.camunda.bpm.engine.RepositoryService;
import org.camunda.bpm.engine.RuntimeService;
import org.camunda.bpm.engine.TaskService;
import org.camunda.bpm.engine.repository.Deployment;
import org.camunda.bpm.engine.repository.ProcessDefinition;
import org.camunda.bpm.engine.runtime.ProcessInstance;
import org.camunda.bpm.engine.task.Task;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 一个常规的说HaloWode的流程
 * k
 *
 * @author chenchuancheng
 * @since 2022/11/21 10:08
 */
@RestController
public class ProcessController {

    @Resource
    private RepositoryService repositoryService;

    @Resource
    private RuntimeService runtimeService;

    @Resource
    private TaskService taskService;

    @GetMapping("/deploy")
    public String deploy() {
        Deployment deploy = repositoryService.createDeployment()
                .name("流程")
                .addClasspathResource("process.bpmn")
                .deploy();

        return deploy.getId();
    }

    @GetMapping("/queryProcess/{deployId}")
    public List<String> queryProcess(@PathVariable("deployId") String deployId) {
        List<ProcessDefinition> list = repositoryService.createProcessDefinitionQuery()
                .deploymentId(deployId)
                .list();
        return list.stream().map(ProcessDefinition::getId).collect(Collectors.toList());
    }

    @GetMapping("/start/{processDefinitionId}")
    public String start(@PathVariable("processDefinitionId") String processDefinitionId) {
        ProcessInstance processInstance = runtimeService
                .startProcessInstanceById(processDefinitionId);
        return processInstance.getId();
    }

    @GetMapping("/queryTask/{processInstanceId}")
    public List<String> queryTask(@PathVariable("processInstanceId") String processInstanceId) {
        List<Task> list = taskService.createTaskQuery()
                .processInstanceId(processInstanceId)
                .list();
        return list.stream().map(Task::getId).collect(Collectors.toList());
    }

    @GetMapping("/finishTask/{taskId}")
    public void finishTask(@PathVariable("taskId") String taskId) {
        taskService.complete(taskId);
    }

}
